;=================================================================
; TRESPASSER CLASSES
;=================================================================
; EDIT: This documentation may not accurately describe its content.
;
; This text is a compilation of Trespasser classes re-built
; as FASM data definitions. The class names are the exact names
; that would have been used in the original Trespasser source
; (with 1.1 patch, if any changed). However, the class
; members do not originate from the source and are named
; based on my best estimation of their usage, which is
; in turn based on real-time and disassembly-based study
; of the class structures in memory. Members that do not
; have labels typically have unknown or partially determined uses.
; Members should be the same for all 1.1 executables.
;
; Every Trespasser class is associated with a certain Virtual Table,
; which I will henceforth label VTable, which is a pointer to a list of
; addresses which define the methods (functions) used by the class. This list
; is static and always found in the EXE's data section, unless intercepted
; by ATX. It is always the first member of the class and can practically
; be used to identify the class type in real-time (as it only changes
; during initialization). There are one or two exceptions, that is, classes
; that have 2 possible VTables, but it is rare and can generally be disregarded.
; I have only noticed one case, CUIListBox, and in this case the two VTables
; are both used, but placed at two different areas of the class.
; In this file, only some VTables are defined in structures, not all. They are
; labeled as [ParentClassName]VTable.
;
; Method calls in Trespasser follow standard C++ calling convention
; (param 1 = left param = lowest in memory), but the instance upon which
; the method acts upon (the "this" object pointer) is always passed
; in the ecx register and NOT as the first parameter on the stack.
; Parameters are usually DWORDs and are assumed to be unless otherwise noted.
; Return values (if any) are returned in eax/al or in the st0 fpu register
; (if a float is returned).
;
; Many thanks to Tomasz Grysztar for the help with the structure definitions. Also,
; many of the example macros (from FASM.PDF v1.64 documentation) have been
; re-used in the "customstrucs.inc" file; I will take no credit for those.
;

;-------------------------------
; SYNTAX
;-------------------------------
; - "customstruc" defines the class or VTable structure name.
; - "endcs" closes the last defined class or VTable structure.
; - "member" instruction is used to offset the struc member on the following
; line by the specified amount of bytes from the start of the structure.
; It must be used in order and offsets must be equal to or greater than accumulated
; definition size from after last member, or will not compile (will give "invalid value" error).
; It is not mandatory, but is used often here. The member types themselves are often
; define as "dd", which is short for 32-bit DWORD, which can represent a 32-bit float,
; integer, pointer, boolean, etc. Hints are provided in the member names, but usually
; values that are taken directly from T-Scripts will have the same types (float, bool, int).
; - "strucsize" has the same behaviour as "member", but is used at the end of
; structures to ensure the structure has a certain length (sizeof.[strucname]),
; which is useful if the last member(s) of the structure is/are not defined
; or is/are unknown. All structure sizes can be accessed via the "sizeof.strucname"
; instruction usually defined by the "struct" macro. In this case, "strucsize"
; determines the classe's length, in bytes.
;
; - All non-class structures used to define member types (such as
; LOCATION and ORIENTQUAT) are defined in "customstrucs.inc". All
; custom macros relating directly to these custom structure definitions
; can also be found in that file.

customstruc CInstance				; all level-based instances inherit this class at offset 0
	    .lpVTable		    dd ?
     member 0028h,.lpCInfo,	    dd ?
     member 002Ch,.InstanceNameID,  dd ?
     member 0030h,.Location,	    LOCATION
     member 003Ch,.OrientationQuat, ORIENTQUAT
     strucsize 050h
endcs

customstruc CEntity
   ;inherits: CEntity, CInstance, CPartition, CFetchable
	    .				  CInstance
endcs

;   ;VTable Methods (125 total)
;   customstruc ICEntity
;               .Release                dd ?
;        member 01A4h,.CMessageDamageHandler,        dd ?
;        member 01ACh,.CMessageMoveHandler,          dd ?
;        member 01B0h,.CMessageDeleteHandler,        dd ?
;        member 01B4h,.CMessageCreateHandler,        dd ?
;        member 01BCh,.CMessageDeathHandler,         dd ?
;        member 01C0h,.CMessageUseHandler,           dd ?
;        member 01C4h,.CMessagePickUpHandler,        dd ?
;        member 01C8h,.CMessagePaintHandler,         dd ?
;        member 01D0h,.CMessageTriggerHandler,       dd ?
;        member 01D4h,.CMessageCollisionHandler,     dd ?
;        member 01DCh,.CMessagePhysicsReqHandler,    dd ?
;        member 01E0h,.CMessageAudioHandler,         dd ?
;        member 01ECh,.CMessageControlHandler,       dd ?
;        member 01F0h,.CMessageStepHandler,          dd ?
;        strucsize 500
;   endcs


customstruc CAnimate
	    .				  CInstance
     member 0050h,.HitPoints,		  dd ?
     member 0054h,.MaxHitPoints,	  dd ?
     member 00ACh,.lpCBoundaryBoxBody,	  dd ?	  ; note: All CBoundaryBox objects are of the same class
     member 00B0h,.lpCBoundaryBoxFoot,	  dd ?	  ; Boundary box locations are only updated and valid after the dino
     member 00B4h,.lpCBoundaryBoxHand,	  dd ?
     member 00B8h,.lpCBoundaryBoxHead,	  dd ?	  ; is first activated
     member 00BCh,.lpCBoundaryBoxTail,	  dd ?
endcs

customstruc CAnimal
	    .				  CInstance
     member 0050h,.HitPoints,		  dd ?
     member 0054h,.MaxHitPoints,	  dd ?
     member 00ACh,.lpCBoundaryBoxBody,	  dd ?	  ; note: All CBoundaryBox objects are of the same class
     member 00B0h,.lpCBoundaryBoxFoot,	  dd ?	  ; Boundary box locations are only updated and valid after the dino
     member 00B8h,.lpCBoundaryBoxHead,	  dd ?	  ; is first activated
     member 00BCh,.lpCBoundaryBoxTail,	  dd ?
     member 00DCh,.lpCBrain,		  dd ?
     strucsize 0E0h
endcs

customstruc CPlayerPriv
	    .				  CInstance
     member 0050h,.HitPoints,		  dd ?
     member 0054h,.MaxHitPoints,	  dd ?
endcs

customstruc CBoundaryBox
	    .				  CInstance
     member 0050h,.lpParentInstance,	  dd ?
     strucsize 060h
endcs


customstruc CGun
	    .			     CInstance
     member 0068h,.MaxAmmoCount,     dd ?
     member 006Ch,.AmmoCount,	     dd ?
     member 00D8h,.lpCMuzzleFlash1,  dd ?
     member 00DCh,.lpCMuzzleFlash2,  dd ?
     member 00E0h,.lpCMuzzleFlash3,  dd ?
     strucsize 0104h
endcs

customstruc CInputDeemone
	    .					     CEntity
     member 0080h,.GameInputsEnabled,		     db ?
     strucsize 0088h
endcs


customstruc CAISystem
	    .lpVTable				     dd ?
     member 31B4h,.lpLastActiveBrain,		     dd ?
     member 31B8h,.lpLastActiveSynthesizer,	     dd ?
     strucsize 31D0h
endcs

customstruc CBrain
	    .lpVTable				     dd ?
	    .lpCMentalState			     dd ?
	    .lpCWorldView			     dd ?
     member 0010h,.lpCActivityCompound, 	     dd ?
     member 0014h,.lpCActivityEat,		     dd ?
     member 0018h,.lpCActivityBite,		     dd ?
     member 001Ch,.lpCActivityFeint,		     dd ?
     member 0020h,.lpCActivityRam,		     dd ?
     member 0024h,.lpCActivityDrink,		     dd ?
     member 0028h,.lpCActivity1,		     dd ?
     member 002Ch,.lpCActivity2,		     dd ?
     member 0030h,.lpCActivityTailSwipe,	     dd ?
     member 0034h,.lpCActivity3,		     dd ?
     member 0038h,.lpCActivity4,		     dd ?
     member 003Ch,.lpCActivity5,		     dd ?
     member 0040h,.lpCActivityVocalSnarl,	     dd ?
     member 0044h,.lpCActivityVocalCroon,	     dd ?
     member 0048h,.lpCActivity6,		     dd ?
     member 004Ch,.lpCActivityMoveCommandToward,     dd ?
     member 0050h,.lpCActivity7,		     dd ?
     member 0054h,.lpCActivity8,		     dd ?
     member 0058h,.lpCActivity9,		     dd ?
     member 005Ch,.lpCActivityMoveHintWander,	     dd ?
     member 0060h,.lpCActivity10,		     dd ?
     member 0064h,.lpCActivity11,		     dd ?
     member 0068h,.lpCActivityMoveCommandStayNear,   dd ?
     member 006Ch,.lpCActivityMoveCommandStayAway,   dd ?
     member 0070h,.lpCActivityFlee,		     dd ?
     member 0074h,.lpCActivityPursue,		     dd ?
     member 0078h,.lpCActivityJumpBack, 	     dd ?
     member 007Ch,.lpCActivityGetOut,		     dd ?
     member 0080h,.lpCActivityApproach, 	     dd ?
     member 0084h,.lpCActivity12,		     dd ?
     member 0088h,.lpCActivityMoveBy,		     dd ?
     member 008Ch,.lpCActivity13,		     dd ?
     member 0090h,.lpCActivity14,		     dd ?
     member 0094h,.lpCActivitySniffTarget,	     dd ?
     member 0098h,.lpCActivityCockHead, 	     dd ?
     member 009Ch,.lpCActivityRearBack, 	     dd ?
     member 00A0h,.lpCActivity15,		     dd ?
     member 00A4h,.lpCActivityCower,		     dd ?
     member 00A8h,.lpCActivityGlare,		     dd ?
     member 00ACh,.lpCActivitySniffAir, 	     dd ?
     member 00B0h,.lpCActivityLookAround,	     dd ?
     member 00B4h,.lpCActivityNothing,		     dd ?
     member 00B8h,.lpCActivity16,		     dd ?
     member 00BCh,.lpCActivity17,		     dd ?
     member 00C0h,.lpCActivity18,		     dd ?
     member 00C4h,.lpCActivity19,		     dd ?
     member 00C8h,.lpCActivity20,		     dd ?
     member 00CCh,.lpCActivityDOSubBrain1,	     dd ?
     member 00D0h,.lpCActivityDOSubBrain2,	     dd ?
     member 00D4h,.lpCActivityDOSubBrain3,	     dd ?
     member 00D8h,.lpCActivityDOSubBrain4,	     dd ?
     member 00E0h,.lpCSynthesizer,		     dd ?
     member 011Ch,.lpCAnimal1,			     dd ?
     member 0128h,.CMessagePhysicsReqVTable,	     dd ?
     member 0130h,.lpCAnimal2,			     dd ?
     member 0140h,label .StringName1
     member 01E8h,.lpCValueTable,		     dd ?
     member 01ECh,.lpCObjectValue,		     dd ?
     member 0220h,label .ParentInstanceNameString
     member 0264h,.Bravery,			     dd ?
     member 026Ch,.HumanFear			     dd ?
     member 0270h,.HumanLove			     dd ?
     member 0274h,.HumanAnger			     dd ?
     member 0278h,.HumanCuriosity		     dd ?
     member 027Ch,.HumanHunger			     dd ?
     member 0280h,.HumanThirst			     dd ?
     member 0284h,.HumanFatigue 		     dd ?
     member 0288h,.HumanPain			     dd ?
     member 028Ch,.HumanSolidity		     dd ?
     member 0290h,.DamageFear			     dd ?
     member 0294h,.DamageLove			     dd ?
     member 0298h,.DamageAnger			     dd ?
     member 029Ch,.DamageCuriosity		     dd ?
     member 02A0h,.DamageHunger 		     dd ?
     member 02A4h,.DamageThirst 		     dd ?
     member 02A8h,.DamageFatigue		     dd ?
     member 02ACh,.DamagePain			     dd ?
     member 02B0h,.DamageSolidity		     dd ?
     member 0340h,.WakeUp,			     dd ?
     member 0344h,.Sleep,			     dd ?
     strucsize 894
endcs

customstruc CInfluence
	    .lpVTable				     dd ?
     member 033Ch,.lpSourceInstance,		     dd ?	; for ActBite, this is the bite "target"
endcs

customstruc CMentalState
	    .lpVTable			   dd ?
     member 0004h,.Fear,		   dd ?
     member 0008h,.Love,		   dd ?
     member 000Ch,.Anger,		   dd ?
     member 0010h,.Curiosity,		   dd ?
     member 0014h,.Hunger,		   dd ?
     member 0018h,.Thirst,		   dd ?
     member 001Ch,.Fatigue,		   dd ?
     member 0020h,.Pain,		   dd ?
     member 0024h,.Solidity,		   dd ?
endcs

customstruc CActivity
	    .lpVTable				     dd ?
     member 0034h,.StateFlags,			     dd ?
     member 004Ch,label .ActivityName					; 16 bytes, including null-terminator
     strucsize 005Ch
endcs

;flags for member CActivity.StateFlags
     ACTIVITY_ENABLED				     equ	 4h
     ACTIVITY_FLAG_UNKNOWN			     equ       200h	; ??? (maybe "in progress" or "active"?)

customstruc CActivityMoveCommandStayNear
	    .lpVTable				     dd ?
     member 005Ch,.StayNearTargetValueID32,	     dd ?		; hash of target instance
     member 0064h,.StayNearMaxValue,		     dd ?
     member 0068h,.StayNearOKValue,		     dd ?
     strucsize 06Ch
endcs

customstruc CInfo
	    .lpVTable			   dd ?
     member 0008h,.lpCRenderType,	   dd ? 	; for type 2 geometry instances, will be a CMesh instance ***
     member 000Ch,.lpCPhysicsInfo,	   dd ?
     member 0010h,.lpCAIInfo,		   dd ?
endcs
 ; *** note: CMesh instance is shared between instances of same class
 ;     (cannot store current animation state info, only static mesh/bones/joints/anim data, depending on mesh type)

customstruc CMessage
   ;inherits: CMessage
	    .lpVTable		    dd ?
endcs

customstruc CAudioDaemon
	    .lpVTable				dd ?
     member 00C4h,.lpTPAStructEffects,		dd ?
     member 00C8h,.lpTPAStructStream,		dd ?
     member 00CCh,.lpTPAStructAmbient,		dd ?
endcs



;--------------------------------
;ACTIONTYPE CLASSES
;--------------------------------

customstruc CAction
	    .lpVTable		    dd ?
	    .refCount		    dd ?
endcs

customstruc CVoiceOverAction		; ActionType 00
	    .lpVTable		    dd ?
	    .RefCount		    dd ?
endcs

customstruc CAmbientAction		; ActionType 01
	    .lpVTable		    dd ?
	    .RefCount		    dd ?
endcs

customstruc CMusicAction		; ActionType 02
	    .lpVTable		    dd ?
	    .RefCount		    dd ?
endcs

customstruc CFadeMusicAction		; ActionType 03
	    .lpVTable		    dd ?
	    .RefCount		    dd ?
endcs

customstruc CShowOverlayAction		; ActionType 04
	    .lpVTable		    dd ?
	    .RefCount		    dd ?
endcs

customstruc CSetFogAction		; ActionType 05
	    .lpVTable		    dd ?
	    .RefCount		    dd ?
	    .Flags		    dd ?	; combination of flags define above
	    .RGBColor24Bit	    dd ?
	    .FogPower		    dd ?
	    .FogHalf		    dd ?
     strucsize 0018h
endcs

; flags for member CSetFogAction.Flags
     CSETFOG_USE_RGB	       equ	   1
     CSETFOG_USE_FOGTYPE       equ	   2
     CSETFOG_USE_FOGPOWER      equ	   4
     CSETFOG_USE_FOGHALF       equ	   8
  ; these two flags cannot be combined and should only be used if CSETFOG_USE_FOGTYPE is used
     CSETFOG_FOGTYPE_0	       equ	   10h
     CSETFOG_FOGTYPE_1	       equ	   20h

customstruc CSetRendererAction		; ActionType 06
	    .lpVTable		    dd ?
	    .RefCount		    dd ?
endcs

customstruc CSetTerrainAction		; ActionType 07
	    .lpVTable		    dd ?
	    .RefCount		    dd ?
endcs

customstruc CSetImageCacheAction	; ActionType 08
	    .lpVTable		    dd ?
	    .RefCount		    dd ?
endcs

customstruc CSetAIAction		; ActionType 09
	    .lpVTable		    dd ?
	    .RefCount		    dd ?
endcs

customstruc CSetPhysicsAction		; ActionType 10
	    .lpVTable			   dd ?
	    .RefCount			   dd ?
     member 0008h,.Velocity,		   VELOCITY
     member 0014h,.FrozenBOOL,		   db ?
     member 0018h,.TargetInstance,	   dd ?
     member 001Ch,.ImpulseBOOL, 	   db ?
     member 0020h,.UnknownLocationSet,	   LOCATION
     member 002Ch,.Facing,		   FACINGVECTOR
     member 0038h,.Push,		   dd ?
     member 003Ch,.EmitterInstance,	   dd ?
     strucsize 0040h
endcs

customstruc CSubstituteMeshAction	; ActionType 11
	    .lpVTable		    dd ?
	    .RefCount		    dd ?
endcs

customstruc CSetSortAction		; ActionType 12
	    .lpVTable		    dd ?
	    .RefCount		    dd ?
endcs

customstruc CSetSkyAction		; ActionType 13
	    .lpVTable		    dd ?
	    .RefCount		    dd ?
endcs

customstruc CSetAlphaWaterAction	; ActionType 14
	    .lpVTable		    dd ?
	    .RefCount		    dd ?
endcs

customstruc CWaterOnOffAction		; ActionType 15
	    .lpVTable		    dd ?
	    .RefCount		    dd ?
endcs

customstruc CLoadLevelAction		; ActionType 16
	    .lpVTable		    dd ?
	    .RefCount		    dd ?
	    .LevelNameLength	    dd ?	; (start of CStringDesc structure)
	    .Flags		    dd ?
	    .LevelName15	    dd 4 dup ?	; used by the game if level name is 15 bytes or less (excluding null-terminator)
     member 0020h,.lpLevelName,     dd ?	; if longer, extra space is allocated for the name
						; this is the actual pointer to the levelname string
     strucsize 0024h
endcs

customstruc CSetAnimatePropertiesAction       ; ActionType 17
	    .lpVTable		    dd ?
	    .RefCount		    dd ?
endcs

customstruc CTeleportAction		; ActionType 18
	    .lpVTable				   dd ?
	    .RefCount				   dd ?
	    .TargetInstanceNameID		   dd ?
     member 000Ch,.TargetInstanceLocationSet,	   LOCATION
     member 0018h,.TargetInstanceOrientationSet,   ORIENTQUAT
     member 0028h,.HeightRelativeBOOL,		   db ?
     member 0029h,.OnTerrainBOOL,		   db ?
     member 002Ah,.SetPositionBOOL,		   db ?
     member 002Bh,.SetOrientationBOOL,		   db ?
     strucsize 002Ch
endcs

customstruc CSaveLevelAction		; ActionType 19
	    .lpVTable		    dd ?
	    .RefCount		    dd ?
	    .LevelNameLength	    dd ?
	    .Flags		    dd ?
	    .LevelName15	    dd 4 dup ? ; used by the game if level name is 15 bytes or less (excluding null-terminator)
     member 0020h,.lpLevelName,     dd ?       ; if longer, extra space is allocated for the name
					       ; this is the actual pointer to the levelname string (which is the text that appears in "Load Level" dialog)
     strucsize 0024h
endcs

customstruc CMagnetAction		; ActionType 20
	    .lpVTable		    dd ?
	    .RefCount		    dd ?
endcs

customstruc CAnimateTextureAction	; ActionType 21
	    .lpVTable		    dd ?
	    .RefCount		    dd ?
endcs

customstruc CHideShowAction		; ActionType 22
	    .lpVTable		    dd ?
	    .RefCount		    dd ?
endcs

customstruc CSoundEffectAction		; ActionType 23
	    .lpVTable		    dd ?
	    .RefCount		    dd ?
endcs

customstruc CWakeAIAction		; ActionType 24
	    .lpVTable		    dd ?
	    .RefCount		    dd ?
endcs

customstruc CDelayAction		; ActionType 25
	    .lpVTable		    dd ?
	    .RefCount		    dd ?
endcs

customstruc CScriptedAnimationAction	; ActionType 26
	    .lpVTable		    dd ?
	    .RefCount		    dd ?
	    .lpAnimsSubStruct	    dd ?
     strucsize 000Ch
endcs

customstruc CSetVariableTriggerAction	; ActionType 27
	    .lpVTable		    dd ?
	    .RefCount		    dd ?
endcs

customstruc CSetHintAction		; ActionType 28
	    .lpVTable		    dd ?
	    .RefCount		    dd ?
endcs

customstruc CAudioEnvironmentAction	; ActionType 29
	    .lpVTable		    dd ?
	    .RefCount		    dd ?
endcs

customstruc CSubstituteAIAction 	; ActionType 30
	    .lpVTable		    dd ?
	    .RefCount		    dd ?
endcs

customstruc CEndGameAction		; ActionType 31
	    .lpVTable		    dd ?
	    .RefCount		    dd ?
endcs

customstruc CControlPlayerAction	; ActionType 32
	    .lpVTable		    dd ?
	    .RefCount		    dd ?
endcs

customstruc CAISystemAction		; ActionType 33
	    .lpVTable		    dd ?
	    .RefCount		    dd ?
endcs

customstruc CTextAction 		; ActionType 34
	    .lpVTable		    dd ?
	    .RefCount		    dd ?
endcs

customstruc CWaterDisturbanceAction	; ActionType 35
	    .lpVTable		    dd ?
	    .RefCount		    dd ?
endcs


customstruc CTrigger
	    .					CInstance
     member 00A8h,.CurrentFireCount		dd ?	      ; is backwards counter for trigger's fire count (original value may be lost)
     member 00ACh,.CurrentProbability		dd ?	      ; fire probability, may be -1 or 0-100, is often set to 0 after fire count expires (original value may be lost)
     member 00B8h,.FireAtZero			db ?
     member 00BCh,.OrigFireCount		dd ?	      ; ONLY set if ResetFire and FireAtZero were set for trigger
endcs


customstruc CMesh
	    .lpVTable				dd ?
     member 0004h,.RefCount			dd ?
     member 0008h,.OtherMesh1			dd ?
     member 0010h,.OtherMesh2			dd ?
     member 0018h,.PolygonStructCount		dd ?
     member 001Ch,.lpPolygonStructs		dd ?	      ; array
     member 0020h,.VertexPointerCount		dd ?
     member 0024h,.lpVertexPointers		dd ?	      ; array
     member 0028h,.VertexStructCount		dd ?
     member 002Ch,.lpVertexStructs		dd ?	      ; array
     member 0030h,.VertexCoordinatesCount	dd ?	      ; (points)
     member 0034h,.lpVertexCoordinates		dd ?	      ; array
     member 0038h,.TextureCount 		dd ?
     member 003Ch,.lpTexturePointers		dd ?	      ; array
     member 0040h,.OtherVertexCoordinatesCount	dd ?
     member 0044h,.lpOtherVertexCoordinates	dd ?
     member 004Ch,.OtherMesh3			dd ?
     member 0050h,.lpCBoundVolBoxVTable 	dd ?
     strucsize 0080h	       ; (or 0084h???)
endcs

customstruc CMeshAnimating
	    .					CMesh
     member 0088h,.AnimSubMaterialIndex 	dd ?	      ; same as (AnimSubMaterial-1) (from int level script value)
     member 00A8h,.NumAnimTextures		dd ?
     member 00ACh,.lpAnimCTextureList		dd ?	      ; points to a list of CTexture instance pointers, number of which is in NumAnimTextures
endcs


customstruc CBioMesh
	    .					CMeshAnimating
     member 00C0h,.NumJointVertexEntries	dd ?	      ; this is the highest "int Axx = ###" ### value of all $J objects of this mesh +1
     member 00C4h,.lpJointVertexEntries 	dd ?	      ; array of ints, 1 per NumJointVertexEntries (each int is a $J object number, i.e., ## in "$JRaptorB##")
     member 00C8h,.NumJoints			dd ?	      ; (excluding double joints)
     member 00CCh,.NumAnim00			dd ?	      ; typically, same as NumDoubleJoints if each double joint has an Anim00 (or not?)
     member 00D0h,.lpAnim00			dd ?	      ; array of single ints (Anim00)
     member 00D4h,.NumAnim01			dd ?	      ; typically, same as NumDoubleJoints if each double joint has an Anim01 (or not?)
     member 00D8h,.lpAnim01			dd ?	      ; array of single ints (Anim01)
     member 00DCh,.NumRatio			dd ?	      ; typically, same as NumDoubleJoints if each double joint has Ratio (or not?)
     member 00E0h,.lpRatio			dd ?	      ; array of single floats (Ratio)
     member 00E4h,.NumRotationRatio		dd ?	      ; typically, same as NumDoubleJoints if each double joint has an RotationRatio (or not?)
     member 00E8h,.lpRotationRatio		dd ?	      ; array of single floats (RotationRatio)
     member 00ECh,.NumJointsTotal		dd ?	      ; NumJoints+NumDoubleJoints
     member 00F0h,.lpJointIDs			dd ?	      ; array of DWORD hashed ID values, one per joint (for all joints)
     strucsize 00F4h	;(or 00F8?)
endcs

; default Anim00 and Anim01 values are 0

; joint #/pointers from +0A8h to +0F0h

; lpVertexJointEntries is indexed by the ### value from "int Axx = ###". The int values
; contained are the numbers for the joint (i.e., ## from "$JRaptorB##")
; -> there can multiple vertices for one joint (A00, A01, etc.), but only
;    one joint for each vertex (values ### from "int Axx = ###" must be
;    unique and appear only once across all $J objects for a mesh)

customstruc CRasterMem	      ; this basically describes the CTexture individual mipmaps (the textures themselves, since CTexture is higher-level)
	    .lpVTable				dd ?
     member 0004h,.RefCount			dd ?
     member 0008h,.Width			dd ?
     member 000Ch,.Height			dd ?
     member 0010h,.BitDepth			dd ?
     member 0014h,.PixelsPerScanline		dd ?	 ; ***
     member 0018h,.lpColorData			dd ?	 ; this pointer is valid ONLY when this CRasterMem instance is the current active bitmap (otherwise, its memory is swapped out of memory, though the pointer stays)
     member 001Ch,.WidthFloat			dd ?	 ; unknown. usually same as Width but in float format (32-bit)
     member 0020h,.HeightFloat			dd ?	 ; unknown. usually same as Height but in float format (32-bit)
     member 0024h,.Unknown1			dd ?	 ; unknown. usually null (0.0?).
     member 0028h,.WidthLastPixelOffset 	dd ?	 ; unknown. usually Width-1 (int). Seems to be -1 for animated water textures.
     member 002Ch,.HeightLastPixelOffset	dd ?	 ; unknown. usually Height-1 (int). Seems to be -1 for animated water textures.
     member 0030h,.lpVTableCRasterSurface	dd ?	 ; short sub-class, don't know usage
     member 0034h,.lpCurrentRasterMem		dd ?	 ; always holds pointer to this base CRasterMem instance (useless?)
     member 0038h,.lpVTableCPixelFormat 	dd ?	 ; sub-class inheriting CPixelFormat and thus giving pixel type
     member 003Ch,.lpCPaletteInfoStruct 	dd ?	 ; same structure that can be gotten indirectly from CTexture.lpPaletteInfoGeneral
     strucsize 0058h	      ; (or 38h???)
endcs

 ; *** number of pixels between the start of each scanline in the color data (for the SWP page alignment), i.e., (number of bytes per scanline)*(bits per pixels/8)
 ;     This usually seems to be used to align scanline on a DWORD boundary in memory.

customstruc CRasterWin		; This is used (for one) to represent the main drawing surface(s)
	    .lpVTable				dd ?
     member 0004h,.RefCount			dd ?
     member 0008h,.Width			dd ?
     member 000Ch,.Height			dd ?
     member 0010h,.BitDepth			dd ?
     member 0014h,.PixelsPerScanline		dd ?
     member 004Ch,.lpDDSurfaceBackBuffer	dd ?
     member 0094h,.lpDDSurfacePrimary		dd ?
     strucsize 06A8h	      ; this is huge...
endcs



customstruc CTexture
	    .lpVTable				dd ?
     member 0004h,.RefCount			dd ?
     member 0008h,.lpCRasterMemMipMaps		dd 6 dup ?	; 6 or less pointers to CRasterMem instances, ordered by texture size (greater size first). Lowest pointers are always filled.
     member 0020h,.NumCRasterMemMipMaps 	dd ?		; number of pointers in the lpCRasterMemMipMaps buffer.
     member 002Ch,.lpPaletteInfoGeneral 	dd ?		; holds other info as well?. First member of the target structure is a pointer to a CPaletteInfoStruct instance.
     member 0034h,.DefaultColor 		dd ?		; This is the value from PID Entry.DefaultColor (see Trespasser File Formats reference)
     member 0038h,.TextureID			dd ?		; (same as PID hash value)
     strucsize 003Ch
endcs

customstruc CRenderWnd
	    .lpVTable				dd ?
     member 0060h,.Gamma			dd ?
endcs


customstruc CGameWnd
	    .lpVTable				dd ?
     member 0038h,.lpConsoleCUIEditbox		dd ?
endcs


customstruc CMainWnd
	    .lpVTable				dd ?
     member 0008h,.lpCurrWindowManager		dd ?
endcs

customstruc CUIEditbox
	    .lpVTable				dd ?
endcs

customstruc CUIDlg
	    .lpVTable				dd ?
     member 0030h,.EnabledKeys			db ?
endcs

;------------------------------------
; THE FOLLOWING CLASSES DO NOT HAVE VTABLES OR OFFICIAL NAMES, AND MAY ALSO BE STRUCTURES
;------------------------------------

customstruc CPaletteInfoStruct
     member 0000h,.NumPaletteEntries		dd ?
     member 0004h,.lpPaletteData		dd ?		; points to array of 32-bit RGB (or BGR?) color values (same as SWP palette data)
     member 0018h,.UnknownMipField		dd ?		; This is the value from [PALETTE BLOCK+0x08] (see Trespasser File Formats reference)
     strucsize 0040h						; (max possible size, maybe smaller)
endcs

